"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PactV3 = void 0;
var ramda_1 = require("ramda");
var package_json_1 = require("../../package.json");
var index_node_1 = __importDefault(require("../../native/index.node"));
function displayQuery(query) {
    var pairs = ramda_1.toPairs(query);
    var mapped = ramda_1.flatten(ramda_1.map(function (_a) {
        var key = _a[0], values = _a[1];
        return ramda_1.map(function (val) { return key + "=" + val; }, values);
    }, pairs));
    return ramda_1.join('&', mapped);
}
function displayHeaders(headers, indent) {
    return ramda_1.join("\n" + indent, ramda_1.map(function (_a) {
        var k = _a[0], v = _a[1];
        return k + ": " + v;
    }, ramda_1.toPairs(headers)));
}
function displayRequest(request, indent) {
    var output = [''];
    output.push(indent + "Method: " + request.method + "\n" + indent + "Path: " + request.path);
    if (request.query) {
        output.push(indent + "Query String: " + displayQuery(request.query));
    }
    if (request.headers) {
        output.push(indent + "Headers:\n" + indent + "  " + displayHeaders(request.headers, indent + "  "));
    }
    if (request.body) {
        var body = JSON.stringify(request.body);
        output.push(indent + "Body: " + body.substr(0, 20) + "... (" + body.length + " length)");
    }
    return output.join('\n');
}
function filterMissingFeatureFlag(mismatches) {
    if (process.env.PACT_EXPERIMENTAL_FEATURE_ALLOW_MISSING_REQUESTS) {
        return mismatches.filter(function (m) { return m.type !== 'missing-request'; });
    }
    return mismatches;
}
function extractMismatches(mockServerMismatches) {
    return mockServerMismatches.map(function (mismatchJson) { return JSON.parse(mismatchJson); });
}
function generateMockServerError(mismatches, indent) {
    return __spreadArrays([
        'Mock server failed with the following mismatches:'
    ], mismatches.map(function (mismatch, i) {
        if (mismatch.type === 'request-not-found') {
            return "\n" + indent + i + ") The following request was not expected: " + displayRequest(mismatch.request, indent + "    ");
        }
        if (mismatch.type === 'missing-request') {
            return "\n" + indent + i + ") The following request was expected but not received: " + displayRequest(mismatch.request, indent + "    ");
        }
        return "" + indent + i + ") " + mismatch.type + " " + (mismatch.path ? "(at " + mismatch.path + ") " : '') + mismatch;
    })).join('\n');
}
var PactV3 = /** @class */ (function () {
    function PactV3(opts) {
        this.states = [];
        this.opts = opts;
        this.setup();
    }
    PactV3.prototype.given = function (providerState, parameters) {
        this.states.push({ description: providerState, parameters: parameters });
        return this;
    };
    PactV3.prototype.uponReceiving = function (desc) {
        this.pact.addInteraction(desc, this.states);
        return this;
    };
    PactV3.prototype.withRequest = function (req) {
        this.pact.addRequest(req, req.body);
        return this;
    };
    PactV3.prototype.withRequestBinaryFile = function (req, contentType, file) {
        this.pact.addRequestBinaryFile(req, contentType, file);
        return this;
    };
    PactV3.prototype.withRequestMultipartFileUpload = function (req, contentType, file, part) {
        this.pact.addRequestMultipartFileUpload(req, contentType, file, part);
        return this;
    };
    PactV3.prototype.willRespondWith = function (res) {
        this.pact.addResponse(res, res.body);
        this.states = [];
        return this;
    };
    PactV3.prototype.withResponseBinaryFile = function (res, contentType, file) {
        this.pact.addResponseBinaryFile(res, contentType, file);
        return this;
    };
    PactV3.prototype.withResponseMultipartFileUpload = function (req, contentType, file, part) {
        this.pact.addResponseMultipartFileUpload(req, contentType, file, part);
        return this;
    };
    PactV3.prototype.executeTest = function (testFn) {
        var _this = this;
        var result = this.pact.executeTest(testFn, this.opts);
        if (result.testResult) {
            return result.testResult
                .then(function (val) {
                var testResult = _this.pact.getTestResult(result.mockServer.id);
                if (testResult.mockServerError) {
                    return Promise.reject(new Error(testResult.mockServerError));
                }
                if (testResult.mockServerMismatches) {
                    var mismatches = extractMismatches(testResult.mockServerMismatches);
                    if (filterMissingFeatureFlag(mismatches).length > 0) {
                        // Feature flag: allow missing requests on the mock service
                        return Promise.reject(new Error(generateMockServerError(mismatches, '  ')));
                    }
                }
                _this.pact.writePactFile(result.mockServer.id, _this.opts);
                return val;
            })
                .catch(function (err) {
                var testResult = _this.pact.getTestResult(result.mockServer.id);
                if (testResult.mockServerError || testResult.mockServerMismatches) {
                    var error = 'Test failed for the following reasons:';
                    error += "\n\n  Test code failed with an error: " + err.message;
                    if (err.stack) {
                        error += "\n" + err.stack + "\n";
                    }
                    if (testResult.mockServerError) {
                        error += "\n\n  " + testResult.mockServerError;
                    }
                    if (testResult.mockServerMismatches) {
                        error += "\n\n  " + generateMockServerError(extractMismatches(testResult.mockServerMismatches), '    ');
                    }
                    return Promise.reject(new Error(error));
                }
                return Promise.reject(err);
            })
                .finally(function () {
                _this.pact.shutdownTest(result);
                _this.setup();
            });
        }
        this.pact.shutdownTest(result);
        this.setup();
        return Promise.reject(result.testError);
    };
    // reset the internal state
    // (this.pact cannot be re-used between tests)
    PactV3.prototype.setup = function () {
        this.pact = new index_node_1.default.Pact(this.opts.consumer, this.opts.provider, package_json_1.version, ramda_1.omit(['consumer', 'provider', 'dir'], this.opts));
    };
    return PactV3;
}());
exports.PactV3 = PactV3;
//# sourceMappingURL=pact.js.map